# -*- coding: utf-8 -*-
# Powered by Kanak Infosystems LLP.
# © 2020 Kanak Infosystems LLP. (<https://www.kanakinfosystems.com>).

import json
import logging
import requests
import werkzeug.utils
from werkzeug.exceptions import BadRequest

from odoo import api, http, SUPERUSER_ID
from odoo.http import request
from odoo import registry as registry_get

from odoo.addons.auth_oauth.controllers.main import OAuthLogin as Home, fragment_to_query_string
from odoo.addons.web.controllers.utils import _get_login_redirect_url
from odoo.exceptions import AccessDenied

_logger = logging.getLogger(__name__)


class OAuthLogin(Home):

    def list_providers(self):
        try:
            providers = request.env['auth.oauth.provider'].sudo().search_read([('enabled', '=', True)])
        except Exception:
            providers = []

        for provider in providers:
            return_url = request.httprequest.url_root + 'auth_oauth/signin'
            if provider['microsoft_provider']:
                return_url = request.httprequest.url_root + 'auth_oauth/microsoft/signin'
            auth_endpoint = provider['auth_endpoint']
            state = self.get_state(provider)
            params = dict(
                response_type='code' if provider['microsoft_provider'] else 'token',
                client_id=provider['client_id'],
                redirect_uri=return_url,
                scope=provider['scope'],
                state=json.dumps(state)
            )
            if provider['microsoft_provider']:
                params.update({
                    'prompt': 'select_account',
                })
                if provider.get("microsoft_tenant_id") != 'common':
                    auth_endpoint = provider['auth_endpoint'].replace('common', provider["microsoft_tenant_id"])
            provider['auth_link'] = "%s?%s" % (auth_endpoint, werkzeug.urls.url_encode(params))
        return providers


class OAuthController(http.Controller):

    @http.route('/auth_oauth/microsoft/signin', type='http', auth='none')
    @fragment_to_query_string
    def microsoft_signin(self, **kw):
        state = json.loads(kw['state'])
        dbname = state['d']
        if not http.db_filter([dbname]):
            return BadRequest()
        context = state.get('c', {})
        registry = registry_get(dbname)
        with registry.cursor() as cr:
            try:
                env = api.Environment(cr, SUPERUSER_ID, context)

                provider = env['auth.oauth.provider'].sudo().browse(state['p'])
                authorization_data = provider.sudo().get_microsoft_oauth_token(kw.get("code"), refresh_token=None)
                kw.update(authorization_data)

                try:
                    headers = {
                        "Authorization": authorization_data.get("access_token"),
                        "Accept": "application/json"
                    }
                    response = requests.get(provider.data_endpoint + "/v1.0/me", headers=headers)
                    kw.update(response.json())

                except Exception as e:
                    _logger.exception("Error to fetch the user details: %s" % e)

                kw.update({
                    "user_id": kw.get("id"),
                    "email": kw.get("userPrincipalName"),
                    "name": kw.get("displayName")
                })

                db, login, key = env['res.users'].sudo().microsoft_auth_oauth(provider.id, kw)
                cr.commit()
                action = state.get('a')
                menu = state.get('m')
                redirect = werkzeug.urls.url_unquote_plus(state['r']) if state.get('r') else False
                url = '/web'
                if redirect:
                    url = redirect
                elif action:
                    url = '/web#action=%s' % action
                elif menu:
                    url = '/web#menu_id=%s' % menu

                pre_uid = request.session.authenticate(db, login, key)
                resp = request.redirect(_get_login_redirect_url(pre_uid, url), 303)
                resp.autocorrect_location_header = False

                # Since /web is hardcoded, verify user has right to land on it
                if werkzeug.urls.url_parse(resp.location).path == '/web' and not request.env.user._is_internal():
                    resp.location = '/'
                return resp
            except AttributeError:
                # auth_signup is not installed
                _logger.error("auth_signup not installed on database %s: oauth sign up cancelled." % (dbname,))
                url = "/web/login?oauth_error=1"
            except AccessDenied:
                # oauth credentials not valid, user could be on a temporary session
                _logger.info('OAuth2: access denied, redirect to main page in case a valid session exists, without setting cookies')
                url = "/web/login?oauth_error=3"
                redirect = request.redirect(url, 303)
                redirect.autocorrect_location_header = False
                return redirect
            except Exception as e:
                # signup error
                _logger.exception("OAuth2: %s" % str(e))
                url = "/web/login?oauth_error=2"

        redirect = request.redirect(url, 303)
        redirect.autocorrect_location_header = False
        return redirect
